package com.trafficparrot.examples.jms;

import com.trafficparrot.examples.tools.jms.JmsClient;
import org.assertj.core.api.Assertions;
import org.junit.Test;

import javax.jms.JMSException;
import java.time.Duration;
import java.time.LocalDateTime;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class HelloWorldJmsQueueIntegrationTest {
    private final JmsClient jmsClient = new JmsClient("tcp://localhost:19091");

    @Test(timeout = 30_000)
    public void helloWorldJmsQueue() throws JMSException {
        jmsClient.sendTextMessageToQueue("request.queue", "hello");

        String response = jmsClient.consumeTextMessageFromQueue("response.queue");

        assertThat(response, is("world"));
    }

    @Test(timeout = 30_000)
    public void helloWorldJmsQueueWithFixedDelay() throws JMSException {
        jmsClient.sendTextMessageToQueue("request.queue", "hello-delay");

        LocalDateTime start = LocalDateTime.now();
        String response = jmsClient.consumeTextMessageFromQueue("response.queue");
        LocalDateTime end = LocalDateTime.now();

        Assertions.assertThat(response).isEqualTo("world-delay");
        Assertions.assertThat(Duration.between(start, end)).isGreaterThanOrEqualTo(Duration.ofMillis(999));
    }
}
