package com.trafficparrot.examples.hello.ibmmq;

import com.ibm.mq.*;
import com.ibm.mq.constants.CMQC;

import java.io.IOException;
import java.util.Hashtable;

import static com.ibm.mq.constants.CMQC.*;

public class IbmMqClient {
    private static MQQueueManager queueManager() throws MQException {
        Hashtable<Object, Object> connectionProperties = new Hashtable<>();
        connectionProperties.put(CMQC.HOST_NAME_PROPERTY, System.getProperty("ibm.mq.host", "localhost"));
        connectionProperties.put(CMQC.PORT_PROPERTY, 1414);
        connectionProperties.put(CMQC.CHANNEL_PROPERTY, System.getProperty("ibm.mq.channel", "DEV.APP.SVRCONN"));
        connectionProperties.put(CMQC.USER_ID_PROPERTY, System.getProperty("ibm.mq.user", "app"));
        connectionProperties.put(CMQC.PASSWORD_PROPERTY, System.getProperty("ibm.mq.password", ""));
        return new MQQueueManager(System.getProperty("ibm.mq.qm", "QM1"), connectionProperties);
    }

    public void sendIbmMqMessageToQueue(String queueName, String text) throws MQException, IOException {
        sendIbmMqMessageTo(true, queueName, text);
    }

    public void sendIbmMqMessageToTopic(String topicName, String text) throws MQException, IOException {
        sendIbmMqMessageTo(false, topicName, text);
    }

    private void sendIbmMqMessageTo(boolean queue, String name, String text) throws MQException, IOException {
        MQQueueManager mqQueueManager = queueManager();
        MQDestination mqDestination = mqDestination(queue, name, mqQueueManager);
        MQMessage mqMessage = new MQMessage();
        mqMessage.format = CMQC.MQFMT_STRING;
        mqMessage.writeString(text);
        MQPutMessageOptions mqPutMessageOptions = new MQPutMessageOptions();
        mqPutMessageOptions.options = CMQC.MQPMO_NEW_CORREL_ID | CMQC.MQPMO_NEW_MSG_ID;
        mqDestination.put(mqMessage, mqPutMessageOptions);
        mqDestination.close();
        mqQueueManager.close();
    }

    private MQDestination mqDestination(boolean queue, String destinationName, MQQueueManager mqQueueManager) throws MQException {
        if (queue) {
            return mqQueueManager.accessQueue(destinationName, MQOO_OUTPUT);
        } else {
            return mqQueueManager.accessTopic(destinationName, "", MQTOPIC_OPEN_AS_PUBLICATION, MQOO_OUTPUT);
        }
    }

    public String receiveIbmMqMessageFromQueue(String queueName) throws MQException, IOException {
        MQQueueManager mqQueueManager = queueManager();
        MQQueue mqQueue = mqQueueManager.accessQueue(queueName, MQOO_INPUT_SHARED);
        MQMessage mqMessage = new MQMessage();
        MQGetMessageOptions mqGetMessageOptions = new MQGetMessageOptions();
        mqGetMessageOptions.options = CMQC.MQGMO_WAIT | CMQC.MQGMO_FAIL_IF_QUIESCING;
        mqGetMessageOptions.waitInterval = 10_000;
        mqQueue.get(mqMessage, mqGetMessageOptions);
        String message = mqMessage.readStringOfByteLength(mqMessage.getDataLength());
        mqQueue.close();
        mqQueueManager.close();
        return message;
    }

    public WaitForOneIbmMqMessage listenOnIbmMqTopic(String topicName) throws MQException {
        MQQueueManager mqQueueManager = queueManager();
        MQTopic mqTopic = mqQueueManager.accessTopic(topicName, "", CMQC.MQTOPIC_OPEN_AS_SUBSCRIPTION, CMQC.MQSO_CREATE);
        return new WaitForOneIbmMqMessage(mqQueueManager, mqTopic);
    }
}
