package com.trafficparrot.examples.jms;

import com.trafficparrot.examples.tools.jms.JmsClient;
import com.trafficparrot.examples.tools.jms.WaitForOneJmsMessage;
import org.assertj.core.api.Assertions;
import org.junit.Test;

import javax.jms.JMSException;
import java.time.Duration;
import java.time.LocalDateTime;

import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

public class HelloWorldJmsTopicIntegrationTest {
    private final JmsClient jmsClient = new JmsClient("tcp://localhost:19091");

    @Test(timeout = 30_000)
    public void helloWorldJmsTopic() throws JMSException, InterruptedException {
        WaitForOneJmsMessage waitForOneJmsMessage = jmsClient.listenOnTopic("response.topic");
        jmsClient.sendTextMessageToTopic("request.topic", "hello");

        String response = waitForOneJmsMessage.awaitMessage();

        assertThat(response, is("world"));
    }


    @Test(timeout = 30_000)
    public void helloWorldJmsTopicWithFixedDelay() throws JMSException, InterruptedException {
        WaitForOneJmsMessage waitForOneJmsMessage = jmsClient.listenOnTopic("response.topic");

        jmsClient.sendTextMessageToTopic("request.topic", "hello-delay");

        LocalDateTime start = LocalDateTime.now();
        String response = waitForOneJmsMessage.awaitMessage();
        LocalDateTime end = LocalDateTime.now();

        Assertions.assertThat(response).isEqualTo("world-delay");
        Assertions.assertThat(Duration.between(start, end)).isGreaterThanOrEqualTo(Duration.ofMillis(999));
    }
}
